searchNodes=[{"ref":"Envio.html","title":"Envio","type":"module","doc":"Main interface to Envio. Provides handy functions to publish messages, subscribe to messages, etc."},{"ref":"Envio.html#register/2","title":"Envio.register/2","type":"function","doc":""},{"ref":"Envio.Backend.html","title":"Envio.Backend","type":"behaviour","doc":"The behaviour to be implemented for all the backends."},{"ref":"Envio.Backend.html#c:on_envio/2","title":"Envio.Backend.on_envio/2","type":"callback","doc":"The callback when the envio is received."},{"ref":"Envio.Channels.html","title":"Envio.Channels","type":"module","doc":"Channels storage. It manages all the channels currently existing in the system."},{"ref":"Envio.Channels.html#child_spec/1","title":"Envio.Channels.child_spec/1","type":"function","doc":"Returns a specification to start this module under a supervisor. See Supervisor."},{"ref":"Envio.Channels.html#init/1","title":"Envio.Channels.init/1","type":"function","doc":"Default initialization callback (noop.)"},{"ref":"Envio.Channels.html#register/2","title":"Envio.Channels.register/2","type":"function","doc":"Registers new channel."},{"ref":"Envio.Channels.html#start_link/1","title":"Envio.Channels.start_link/1","type":"function","doc":"Starts a new channels bucket."},{"ref":"Envio.Channels.html#state/0","title":"Envio.Channels.state/0","type":"function","doc":"Get list of active subscriptions."},{"ref":"Envio.InconsistentUsing.html","title":"Envio.InconsistentUsing","type":"exception","doc":"An exception to be thrown when an attempt to use scaffolding is inconsistent. For instance, whether the call to use Envio.Subscriber/1 has no valid GenServer defined."},{"ref":"Envio.Utils.html","title":"Envio.Utils","type":"module","doc":"Multipurpose utils, required for the Envío proper functioning, like: fq_name to construct fully-qualified names for channels to prevent clashes"},{"ref":"Envio.Utils.html#config_value/1","title":"Envio.Utils.config_value/1","type":"function","doc":"Returns the anonymous function that either loads the system environment variable or simply returns the value statically loaded from config file. Examples iex&gt; # config :envio, :binary_value, &quot;FOO&quot; iex&gt; Envio.Utils.config_value(Application.get_env(:envio, :binary_value)).() &quot;FOO&quot; iex&gt; # config :envio, :env_value, {:system, &quot;FOO&quot;} iex&gt; System.put_env(&quot;FOO&quot;, &quot;42&quot;) iex&gt; Envio.Utils.config_value(Application.get_env(:envio, :env_value)).() &quot;42&quot;"},{"ref":"Envio.Utils.html#fq_name/2","title":"Envio.Utils.fq_name/2","type":"function","doc":"Produces a fully-qualified name out of namespace in a form of a module name atom and a method name (atom or binary.) Examples iex&gt; Envio.Utils.fq_name(&quot;foo&quot;) &quot;foo&quot; iex&gt; Envio.Utils.fq_name({Foo.Bar, &quot;baz&quot;}) &quot;foo/bar.baz&quot; iex&gt; Envio.Utils.fq_name(Foo.Bar, &quot;baz&quot;) &quot;foo/bar.baz&quot; iex&gt; Envio.Utils.fq_name(Foo.Bar, :baz) &quot;foo/bar.baz&quot; iex&gt; Envio.Utils.fq_name(&quot;Foo.Bar&quot;, &quot;baz&quot;) &quot;Foo.Bar.baz&quot;"},{"ref":"Envio.Utils.html#get_delete/3","title":"Envio.Utils.get_delete/3","type":"function","doc":"Fetches the value from the map by given key and returns both the value and the map without this key. Examples iex&gt; Envio.Utils.get_delete(%{foo: :bar, baz: 42}, :foo) {:bar, %{baz: 42}} iex&gt; Envio.Utils.get_delete(%{baz: 42}, :foo) {nil, %{baz: 42}} iex&gt; Envio.Utils.get_delete(%{baz: 42}, :foo, :bar) {:bar, %{baz: 42}}"},{"ref":"Envio.Utils.html#smart_to_binary/1","title":"Envio.Utils.smart_to_binary/1","type":"function","doc":"Safely converts any term to binary. Examples iex&gt; Envio.Utils.smart_to_binary(&quot;foo&quot;) &quot;foo&quot; iex&gt; Envio.Utils.smart_to_binary(42) &quot;42&quot; iex&gt; Envio.Utils.smart_to_binary(%{foo: :bar, baz: 42}) &quot;%{baz: 42, foo: :bar}&quot; iex&gt; Envio.Utils.smart_to_binary([foo: :bar, baz: 42]) &quot;[foo: :bar, baz: 42]&quot;"},{"ref":"Envio.Publisher.html","title":"Envio.Publisher","type":"behaviour","doc":"Publisher helper scaffold. Simply use Envio.Publisher in the module that should publish messages. The broadcast/2 function becomes available. If the optional channel: argument is passed to use Envio.Publisher, this channel is considered the default one and broadcast/1 function appears to publish directly to the default channel. The ready-to-copy-paste example of usage would be: defmodule MyPub do use Envio.Publisher, channel: :main def publish(channel, what), do: broadcast(channel, what) def publish(what), do: broadcast(what) end All the subscribers of the particular channel the message was published to, will either receive a message (in the case of :pub_sub) or called back with the function provided on subscription (:dispatch). Since v0.8.0 Envío supports Phoenix.PubSub for distributed message broadcasting. The publisher does not wrap :via functionality since it makes not much sense. For how to subscribe, see Envio.Subscriber."},{"ref":"Envio.Publisher.html#c:broadcast/2","title":"Envio.Publisher.broadcast/2","type":"callback","doc":"The callback to publish stuff to Envio."},{"ref":"Envio.Subscriber.html","title":"Envio.Subscriber","type":"behaviour","doc":"Subscriber helper scaffold. To easy register the pub_sub consumer in Envio.Registry, one might use this helper to scaffold the registering/unregistering code. It turns the module into the GenServer and provides the handy wrapper for the respective handle_info/2. One might override handle_envio to implement custom handling. The typical usage would be: defmodule PubSubscriber do use Envio.Subscriber, channels: [{PubPublisher, :foo}] def handle_envio(message, state) do with {:noreply, state} &lt;- super(message, state) do IO.inspect({message, state}, label: &quot;Received message&quot;) {:noreply, state} end end end If channels are not specified as a parameter in call to use Envio.Subscriber, this module might subscribe to any publisher later with subscribe/1: PubSubscriber.subscribe(%Envio.Channel{source: PubPublisher, name: :foo}) For how to publish, see Envio.Publisher."},{"ref":"Envio.Subscriber.html#c:handle_envio/2","title":"Envio.Subscriber.handle_envio/2","type":"callback","doc":"The callback to subscribe stuff to Envio."},{"ref":"Envio.Channel.html","title":"Envio.Channel","type":"module","doc":"Channel description."},{"ref":"Envio.Channel.html#fq_name/1","title":"Envio.Channel.fq_name/1","type":"function","doc":""},{"ref":"Envio.Channel.html#t:t/0","title":"Envio.Channel.t/0","type":"type","doc":"Channel data stored as a struct"},{"ref":"Envio.State.html","title":"Envio.State","type":"module","doc":"Global Envio state. Contains subscriptions, messages that were not yet processed and options."},{"ref":"Envio.State.html#t:t/0","title":"Envio.State.t/0","type":"type","doc":"Internal state of everything amongst Envío."},{"ref":"envio.html","title":"Envío","type":"extras","doc":"Envío Envío is basically a GenEvent², the modern idiomatic pub-sub implementation of event passing. In a nutshell, Envío is a set of handy tools to simplify dealing with Elixir Registry. It includes the instance of Registry to be used out of the box, scaffolds for producing publishers and subscribers. It is built using “convention over configuration” approach, preserving the whole low-level control over the registry entries. Just add the application :envio into your list of extra applications and the default Envio.Registry will be started and managed automagically. def application do [ mod: {MyApplication, []}, extra_applications: ~w|envio ...|a ] end Creating a publisher To create a publisher just use Envio.Publisher in the module that should publish messages to the subscribers. Once use Envio.Publisher is used, broadcast/2 function becomes available. If the optional channel: argument is passed to use Envio.Publisher, this channel is considered the default one and broadcast/1 function appears to publish directly to the default channel. defmodule MyPub do use Envio.Publisher, channel: :main def publish(channel, what), do: broadcast(channel, what) def publish(what), do: broadcast(what) end Another option that might be passed to use Envio.Publisher is manager:, that might be either :registry (default,) or :phoenix_pub_sub to use Phoenix.PubSub for distributed message broadcasting. Creating a subscriber ▶ :dispatch Simply register the handler anywhere in the code: Envio.register( {MySub, :on_envio}, # the function of arity 2 must exist dispatch: %Envio.Channel{source: MyPub, name: :main} ) As MyPub publishes to the :main channel, MySub.on_envio/2 will be called with a message passed as parameter. ▶ :pub_sub Use Envio.Subscriber helper to scaffold the registry subscriber. Implement handle_envio/2 for custom message handling. The default implementation collects last 10 messages in it’s state. This amount might be adjusted by changing :envio, :subscriber_queue_size application environment setting. The implementation below subscribes to :main channel provided by MyPub publisher and prints out each subsequent incoming message to standard output. defmodule PubSucker do use Envio.Subscriber, channels: [{MyPub, :main}] def handle_envio(message, state) do {:noreply, state} = super(message, state) IO.inspect({message, state}, label: &quot;Received&quot;) {:noreply, state} end end ▶ :phoenix_pub_sub Use manager: :phoenix_pub_sub for distributed message broadcasting. The implementation below subscribes to &quot;main&quot; channel in the distributed OTP environment and prints out each subsequent incoming message to standard output. defmodule Pg2Sucker do use Envio.Subscriber, channels: [&quot;main&quot;], manager: :phoenix_pub_sub def handle_envio(message, state) do {:noreply, state} = super(message, state) IO.inspect({message, state}, label: &quot;Received&quot;) {:noreply, state} end end The publisher this subscriber might be listening to would look like defmodule Pg2Spitter do use Envio.Publisher, manager: :phoenix_pub_sub, channel: &quot;main&quot; def spit(channel, what), do: broadcast(channel, what) def spit(what), do: broadcast(what) end"},{"ref":"backends.html","title":"Backends","type":"extras","doc":"Backends Envío supports configurable backends as subscribers with very little to none code. It comes with the following backends out of the box: Slack — a subscriber to envíos that sends all the incoming messages to the preconfigured slack channel IO, ..., more coming Backend is basically a supervised GenServer that implements Envio.Backend behaviour. Backends, provided by the core are generated based on the configuration. The typical config looks like this (example for Envio.Slack): config :envio, :backends, %{ Envio.Slack =&gt; %{ {MyPub, :main} =&gt; [ hook_url: {:system, &quot;SLACK_ENVIO_HOOK_URL&quot;} ] } } Envio.Slack, used as a key, is a name of the module that implements the Envio.Backend behaviour (currently it’s the single function Envio.Backend.on_envio/2 that receives a message when published by the publisher.) The value assigned to the key is a map of {Envio.Publisher, :channel} tuples to the list of arguments that will be injected into the message under :meta key. When the publisher emits, say, %{title: &quot;Pi reminder&quot;, text: &quot;Recall pi value now!&quot;, pi: 3.14} envío, the %{meta: %{hook_url: &quot;my_slack_hook_url&quot;}} will be merged into the message for the further processing by the backend. Typically, the backend will extract this meta information out of the message using {meta, message_without_meta} = Utils.get_delete(message, :meta) and use this information for the backend-specific actions (e. g. what channel to post the message to in the case of Slack.) Implementation details Slack backends understands four predefined keys: title — is treated as title of the message text, message — both are treated as a message body, formatted as pretext level — the message “level” that affects the color of the left bar and the icon. All other keys are treated as values, put as short and long attachments depending on their length (those longer than 32 symbols take the whole line in the slack attachments output.) Slacking using Envío It has never easier. Once your application uses Envío as a publisher, broadcasting messages to the channel named my_app/my_module.main_channel (the latter means messages are published by MyApp.MyModule that has use Envio.Publisher, channel: :main_channel, using broadcast(message), or you have a generic pub_sub publisher registered in Envio.Registry using standard Elixir Registry functions,) simply add the following to the configuration file: config :envio, :backends, %{ Envio.Slack =&gt; %{ {MyApp.MyModule, :main_channel} =&gt; [ # one might simply put a channel name here as binary, # but I don’t recommend that since it’s kinda credentials hook_url: {:system, &quot;SLACK_ENVIO_HOOK_URL&quot;} ] } } Well, that’s it. Your application is now Slack-enabled. Example from Envío tests To test Envío’s backend functionality I used plain old good IO.inspect backend with ExUnit.CaptureIO as a checker: defmodule Envio.IOBackend do @moduledoc false @behaviour Envio.Backend @impl Envio.Backend def on_envio(message, meta) do IO.inspect(message, label: &quot;[★Envío★]&quot;) end end The above is a fully valid backend implementation, that subscribes to the :backends channel of the Spitter.Registry ckass defined in test_helper.exs (the excerpt is from test.exs config): config :envio, :backends, %{ Envio.IOBackend =&gt; %{{Spitter.Registry, :backends} =&gt; []} } Once emitted by Spitter.Registry, this message will be published to the standard output: [★Envío★]: %{bar: 42, meta: %{}} For distributed backend over Phoenix.PubSub.PG2 one might use the following config: config :envio, :backends, %{ Envio.IOBackend.PG2 =&gt; %{&quot;main&quot; =&gt; [manager: :phoenix_pub_sub]} }"}]